#!/usr/bin/perl         

# requires Weigel-RNA!
 use Weigel::Search;
 use Weigel::Search::Filter::MIRNAFilter;

 use Bio::Root::Exception;
 use Error qw(:try);

 my $backend = "Agrep";

 my $search_obj = Weigel::Search->new($backend);	

 # $sbe is now a reference to the backend	
 my $sbe = $search_obj->backend;

 $sbe->settings->tmppath('tmp');
 $sbe->settings->datapath('data');
 
 mkdir($sbe->settings->tmppath);	
 mkdir($sbe->settings->datapath);	

 # generate a Hypa suffix array. you have to do this only once.
 $sbe->generate_database_out_of_fastafile('../t/Test.fasta', 'Description for the test Fastafile');

 my %local_dbs_description = $sbe->get_databases();
 my @local_dbs = sort keys %local_dbs_description;

 # take first available database in our test
 $sbe->settings->database($local_dbs[0]);

 # search for the reverse complement and allow 4 mismatches
 # display 5 bases upstream and downstream of the match
$sbe->settings->query('UGAACAGAAAG');
 $sbe->settings->reverse_complement(1);
 $sbe->settings->mismatches(3);
 $sbe->settings->upstream("5");
 $sbe->settings->downstream("5");
 
 # Vmatch backend is sometimes faster with online(1) when you allow many mismatches
 # $sbe->settings->online(1);

 # filter search results
 my $filter = Weigel::Search::Filter::MIRNAFilter->new();
 # don't delete results, only add a message if filter passed or not
 $filter->delete(0);	
 $sbe->settings->filter( $filter );

 # hypa does not generate alignments, but EMBOSS will automatically some for
 # you. If things should be fast, turn this off	
 # $sbe->settings->no_alignments(1);

  # with the features hash, you can check if the backend supports
  # a special feature. Allow wobble pairs (GU counts 0 or 0.5)	
  if (defined($sbe->features->{GUMISMATCHES})) {
	  $sbe->settings->gumismatches(0.5); 		
  } else {
	print "\nBackend does not support wobble pairs\n";
  }	

 my $start = time;

 # now try to search. we use bioperls Exceptions
 try {
    $sbe->search();
 } catch Bio::Root::SystemException with {
    my $E = shift;
    print STDERR 'Backend call failed: ' . 	
    $E->{'-text'} . ' (' .  $E->{'-line'} . ")\n";
    exit(1);	
 } otherwise {        
	my $E = shift;
    	print STDERR "An unexpected exception occurred: \n$E";
    exit(1);	

 };

 print scalar(@{$sbe->results}) . " search results\n";

 # output all informations we have!
 foreach my $res (@{$sbe->results}) {
    print $res->sequence->id . "\n";
    print $res->mark_subject_uppercase() . "\n";
    print $res->remark . "\n";	
    if (defined($sbe->features->{EVALUE}) &&	defined($sbe->features->{PERCENT_IDENTITY}) ) {
    	print $res->evalue . "\t" . $res->percent_identity . "\n";
    }
   print $res->alignment_string() . "\n\n";

   # get sequences as Bio::SeqIO object and write them in string	
   my $seqio = $sbe->get_sequences([$res->sequence_id]);
   my $string;  my $stringio = IO::String->new($string);
   my $out = Bio::SeqIO->new('-fh' => $stringio,
                             '-format' => 'fasta');

   while ( my $seq = $seqio->next_seq() ) {
      $out->write_seq($seq);
   }
   print $string;
	
 }

 print time - $start . " seconds.\n";
