#!/usr/bin/perl        
 
##############################################
# Just a short tutorial for Weigel::Search
# overview.pl shows some advanced features
##############################################

use strict;
use warnings;

use Weigel::Search;

use Bio::Root::Exception;
use Error qw(:try);

my $backend = "Vmatch";

my $search_obj = Weigel::Search->new($backend);	

# $sbe is now a reference to the backend
# perldoc Weigel::Search::Backend::BackendI	
# perldoc Weigel::Search::Backend::Vmatch
my $sbe = $search_obj->backend;

$sbe->settings->tmppath('tmp');
$sbe->settings->datapath('data');

mkdir($sbe->settings->tmppath);	
mkdir($sbe->settings->datapath);	

# generate a suffix array. you have to do this only once.
$sbe->generate_database_out_of_fastafile('../t/Test.fasta', 'Description for the test Fastafile');

# now choose that database
$sbe->settings->database('Test.fasta');

# search for the reverse complement and allow 2 mismatches
$sbe->settings->query('UGAACAGAAAG');
$sbe->settings->reverse_complement(1);
$sbe->settings->mismatches(2);

# now try to search. we use bioperls Exceptions
try {
   $sbe->search();
} catch Bio::Root::SystemException with {
   my $E = shift;
   print STDERR 'Backend call failed: ' . 	
   $E->{'-text'} . ' (' .  $E->{'-line'} . ")\n";
   exit(1);	
} otherwise {        
   my $E = shift;
   print STDERR "An unexpected exception occurred: \n$E";
   exit(1);	
};

# output some informations! 
# perldoc Weigel::Search::Container::SearchResult
foreach my $res (@{$sbe->results}) {
   print $res->sequence->id . "\n";
   print $res->mark_subject_uppercase() . "\n";
   print $res->alignment_string() . "\n\n";
}

# vim: ft=perl sw=3 ts=3 expandtab
