#!/usr/bin/perl         

 use Weigel::Search::Backends::Vmatch;
 use Weigel::Search::Backends::Agrep;
 use Weigel::Search::Backends::Hypa;
 use Weigel::Search::Filter::MIRNAFilter;

 use Bio::Root::Exception;
 use Error qw(:try);

 my $backend = $ARGV[0];
	
 # configure our search backend, in this case Hypa
 my $backendpackage = "Weigel::Search::Backends::$backend";


 my $sbe = $backendpackage->new();

 $sbe->settings->tmppath('tmp');
 $sbe->settings->datapath('data');
 $sbe->settings->datapath('dat_vmatcha');

 # generate a Hypa suffix array. you have to do this only once.
# $sbe->generate_database_out_of_fastafile('/Users/shenz/merkur_markus/ATH1.cdna', 'AGI Transcripts (- introns, + UTRs)');

 my %local_dbs_description = $sbe->get_databases();
 my @local_dbs = sort keys %local_dbs_description;

 # take first available database in our test
 $sbe->settings->database($local_dbs[0]);

 my $seq = 'UGAACAGAAAGCUCAUGAGCC';

 print "Query: $seq\n";	

 # search for the reverse complement and allow 4 mismatches
 $sbe->settings->query($seq);
 $sbe->settings->reverse_complement(1);
 $sbe->settings->mismatches(3);
 $sbe->settings->upstream("5");
 $sbe->settings->downstream("5");
# $sbe->settings->online(1);

 my $filter = Weigel::Search::Filter::MIRNAFilter->new();
 $filter->delete(0);	
 $sbe->settings->filter( $filter );

 # With many mismatches and short queries, the "online" algorithm
 # is maybe faster. This alogrithm does not use the index. Test this!
#  $sbe->settings->online(1);
#  $sbe->settings->no_alignments(1);
  if (defined($sbe->features->{GUMISMATCHES})) {
#	  $sbe->settings->gumismatches(0);		
	  $sbe->settings->gumismatches(0.5);		
  } else {
	print "\nBackend does not support gu mismatches\n";
  }	

 # now try to search
 my $start = time;
 try {
    $sbe->search();
 } catch Bio::Root::SystemException with {
    my $E = shift;
    print STDERR 'Backend call failed: ' . 	
    $E->{'-text'} . ' (' .  $E->{'-line'} . ")\n";
    exit(1);	
 } otherwise {        
	my $E = shift;
    	print STDERR "An unexpected exception occurred: \n$E";
    exit(1);	

 };

 print scalar(@{$sbe->results}) . " search results\n";
 # output the searchresults with nice alignments
 foreach my $res (@{$sbe->results}) {
    print $res->sequence->id . "\n";
    print $res->mark_subject_uppercase() . "\n";
    print $res->remark . "\n";	
    if (defined($sbe->features->{EVALUE}) &&	defined($sbe->features->{PERCENT_IDENTITY}) ) {
    	print $res->evalue . "\t" . $res->percent_identity . "\n";
    }
   print $res->alignment_string() . "\n\n";

   # get sequences as Bio::SeqIO object and write them in string	
   my $seqio = $sbe->get_sequences([$res->sequence_id]);
   my $string;  my $stringio = IO::String->new($string);
   my $out = Bio::SeqIO->new('-fh' => $stringio,
                             '-format' => 'fasta');

   while ( my $seq = $seqio->next_seq() ) {
      $out->write_seq($seq);
   }
   print $string;
	
 }
 print time - $start . " seconds.\n";
