package Weigel::Search::Filter::FilterI;

use strict;
use warnings;

our $VERSION = '0.1';

use base 'Bio::Root::Root';

use Class::MethodMaker
 [ new => 'new2',
   scalar => [qw / query target message delete / ],
   abstract => [qw / filter / ],
 ];


=head1 NAME

Weigel::Search::Filter::FilterI.pm - Superclass for all filter modules   

=head1 DESCRIPTION

Superclass for all filter modules. Don't use this directly. 
Use this as interface for new filters (see L<Weigel::Search::Filter::MIRNAFilter> for
details). A Filter module implements a filter function that
returns 1 if query and target pass the filter, 0 otherwise.
If the member variable "delete" is 1, then this target won't
be in the search results of the backend.
 

=head1 METHODS

=over 2

=item new()

This function constructs a Filter object. Takes as parameters
two sequences.

=cut

sub new {
   my $self = shift->new2;
   $self->delete(1);
   $self;
}

sub _get_mismatch {
   my ($self, $string1, $string2) = @_ ;    
   my $mismatch =0;    
   for(my $i=0; $i<length($string1) ; $i++){
	   my $char1 = substr($string1,$i,1);	
	   my $char2 = substr($string2,$i,1);
	   if($char1 ne $char2){
	      $mismatch++;	    
	   }
    }
    return $mismatch;   
}

sub _is_mismatch{
    my ($self, $string1, $string2, $position) = @_ ;    
    my $char1 = substr($string1,$position,1);	
    my $char2 = substr($string2,$position,1);
    if($char1 ne $char2){
	return 1;	    
    } else {
        return 0;        
    }   
}

sub _rev_comp {
    my $self = shift;
    my $inString = shift;  
    $inString =~ tr/ACGTacgt/UGCAugca/;
    my $outString;    
    for(my $i=length($inString); $i>=0 ; $i--){
	   my $char = substr($inString,$i,1);	
	   $outString .= $char;	   
	   
       }
    return $outString;    
    
}

=item query()

Get/set the query.

=item target()

Get/set the target.

=item message()

Get/set the message. This is a string with the reason for rejecting the target. 

=item delete()

Get/set delete. If this is 0, then the hit will not be delete from the search results 
(but you will still have the filter message). Otherwise, this hit won't be included in
 the search results of the backend. Default ist 1.

=head1 SEE ALSO

L<Weigel::Search::Backends::Backend>

=head1 AUTHOR

Alexis Maizel, <maizel@tuebingen.mpg.de>,
Markus Riester, <markus.riester@tuebingen.mpg.de>

=cut

1;
# vim: ft=perl sw=3 ts=3 expandtab
